/*
 * LinesField.java
 *
 * Created on November 30, 2001, 3:50 PM
 */

/**
 *
 * @author  jplsp
 */

import java.awt.*;
import java.util.*;

public class LinesField extends javax.swing.JPanel implements java.io.Serializable {
    
    private int[][] field;
    private int[][] searchField;
    private int fieldSize;
    private int nOfColors;
    private Random random;
    private int playState; // init, selected
    private int xSel, ySel;
    private int shortestChain = 5 - 1;
    private LinesManager manager;
    
    public static final Color [] ballColor = {Color.black, Color.red, Color.green,
        Color.yellow, Color.cyan, Color.blue, Color.orange, Color.pink,
        Color.magenta, Color.white, Color.darkGray};
    
    public static final int DefaultNOfColors = 5;
    public static final int DefaultFieldSize = 9;
    
    public static final int PlayStateInit = 0;
    public static final int PlayStateSelected = 1;

    /** Creates new form LinesField */
    public LinesField() {
        random = new Random();
        playState = PlayStateInit;
        initComponents();
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        
        setLayout(new java.awt.BorderLayout());
        
        setPreferredSize(new java.awt.Dimension(450, 450));
        addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                formMousePressed(evt);
            }
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                formMouseClicked(evt);
            }
        });
        
    }//GEN-END:initComponents

    private void formMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMousePressed
        int x, y;
        int dBalls;
        x = evt.getX() / (getWidth() / fieldSize);
        y = evt.getY() / (getHeight() / fieldSize);
        
        switch( playState ){
        case PlayStateInit:
            if (field[x][y] != 0){
                xSel = x;
                ySel = y;
                playState = PlayStateSelected;
                repaint();
            }
            break;
        case PlayStateSelected:
            if (field[x][y] != 0){
                xSel = x;
                ySel = y;
                repaint();
            } else {
                if (attemptMove(xSel, ySel, x, y) == true){
                    field[x][y] = field[xSel][ySel];
                    field[xSel][ySel] = 0;
                    playState = PlayStateInit;
                    dBalls = attemptRemove(x, y);
                    manager.improveScore( dBalls );
                    if (dBalls == 0){
                        if (threeDrop() == false)
                            manager.victory();
                    }
                    repaint();
                }
            }
            break;
        }
    }//GEN-LAST:event_formMousePressed
    
    private void formMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseClicked
        
    }//GEN-LAST:event_formMouseClicked
    
    private boolean attemptMoveNode(int x, int y, int xDest, int yDest){
        if (x == xDest && y==yDest)
            return true;
        if (x<0 || x>=fieldSize || y<0 || y>=fieldSize)
            return false;
        if (searchField[x][y] != 0)
            return false;
        if (field[x][y] != 0)
            return false;
        
        searchField[x][y] = 1;
        
        return (attemptMoveNode(x+1,y,xDest,yDest) ||
                attemptMoveNode(x-1,y,xDest,yDest) ||
                attemptMoveNode(x,y+1,xDest,yDest) ||
                attemptMoveNode(x,y-1,xDest,yDest));
    }

    private boolean attemptMove(int x, int y, int xDest, int yDest){
        int i, j;
        for (i=0; i<fieldSize; i++)
            for (j=0; j<fieldSize; j++)
                searchField[i][j] = 0;
        return (attemptMoveNode(x+1,y,xDest,yDest) ||
                attemptMoveNode(x-1,y,xDest,yDest) ||
                attemptMoveNode(x,y+1,xDest,yDest) ||
                attemptMoveNode(x,y-1,xDest,yDest));
    }

    private int attemptRemove(int x, int y){
        int inf, sup;
        int i;
        int totalRemoved = 0;
        int oldColor = field[x][y];
        
        /* horizontal chain */
        for (inf = x; inf>=0 && field[inf][y] == field[x][y]; inf--)
            ;
        inf ++;
        for (sup = x; sup<fieldSize && field[sup][y] == field[x][y]; sup++)
            ;
        sup --;
        if (sup - inf >= shortestChain){
            totalRemoved += sup - inf;
            for (i=inf; i<=sup; i++)
                field[i][y] = 0;
            field[x][y] = oldColor;
        }

         /* vertical chain */
        for (inf = y; inf>=0 && field[x][inf] == field[x][y]; inf--)
            ;
        inf ++;
        for (sup = y; sup<fieldSize && field[x][sup] == field[x][y]; sup++)
            ;
        sup --;
        if (sup - inf >= shortestChain){
            totalRemoved += sup - inf;
            for (i=inf; i<=sup; i++)
                field[x][i] = 0;
            field[x][y] = oldColor;
        }

        /* backslash chain */
        for (inf = 0; x+inf>=0 && y+inf>=0 && field[x+inf][y+inf] == field[x][y]; inf--)
            ;
        inf ++;
        for (sup = 0; x+sup<fieldSize && y+sup<fieldSize &&
                field[x+sup][y+sup] == field[x][y]; sup++)
            ;
        sup --;
        
        if (sup - inf >= shortestChain){
            totalRemoved += sup - inf;
            for (i=inf; i<=sup; i++)
                field[x+i][y+i] = 0;
            field[x][y] = oldColor;
        }

         /* slash chain */
        
        for (inf = 0; x+inf>=0 && y-inf<fieldSize && field[x+inf][y-inf] == field[x][y]; inf--)
            ;
        inf ++;
        for (sup = 0; x+sup<fieldSize && y-sup>=0 &&
                field[x+sup][y-sup] == field[x][y]; sup++)
            ;
        sup --;
        
        if (sup - inf >= shortestChain){
            totalRemoved += sup - inf;
            for (i=inf; i<=sup; i++)
                field[x+i][y-i] = 0;
        }

        if (totalRemoved > 0){
            totalRemoved++;
            field[x][y] = 0;
        }
        
        return totalRemoved;
    }
     
    public void setManager(LinesManager _manager) {
        manager = _manager;
    }
    
    public void paint(java.awt.Graphics g) {
        int i, j, cWidth, cHeight;
        super.paint(g);
        if (fieldSize !=  0) {
            cWidth = getWidth() / fieldSize;
            cHeight = getHeight() / fieldSize;
            for (i=0; i<fieldSize; i++)
                for (j=0; j<fieldSize; j++) {
                    g.setColor(java.awt.Color.white);
                    g.drawLine(i*cWidth+1, j*cHeight+1, i*cWidth+1, (j+1)*cHeight+1);
                    g.drawLine(i*cWidth+1, j*cHeight+1, (i+1)*cWidth+1, j*cHeight+1);
                    g.setColor(java.awt.Color.black);
                    g.drawLine((i+1)*cWidth, (j+1)*cHeight, i*cWidth, (j+1)*cHeight);
                    g.drawLine((i+1)*cWidth, (j+1)*cHeight, (i+1)*cWidth, j*cHeight);
                    
                    if ( field[i][j] != 0 ){
                        g.setColor( ballColor[field[i][j]] );
                        g.fillOval(i*cWidth+cWidth/7, j*cHeight+cHeight/7,
                            cWidth*5/7, cHeight*5/7);
                    }
                }
            if (playState == PlayStateSelected){
                g.setColor( ballColor[0] );
                g.fillOval(xSel*cWidth+cWidth*4/10, ySel*cHeight+cHeight*4/10,
                    cWidth/5, cHeight/5);
            }
        }
    }
    
    public boolean threeDrop() {
        int freeCells;
        int i, j, k, n, at;
        
        for (k=0; k<3; k++){
            for (i=0, freeCells=0; i<fieldSize; i++)
                for (j=0; j<fieldSize; j++)
                    if (field[i][j] == 0)
                        freeCells++;
            if (freeCells == 0)
                return false;
            at = Math.abs(random.nextInt()) % freeCells;
            for (i=0, n=0; i<fieldSize; i++)
                for (j=0; j<fieldSize; j++){
                    if (n==at && field[i][j] == 0){
                        field[i][j] = Math.abs(random.nextInt()) % nOfColors + 1;
                        attemptRemove(i, j);
                        n++;
                    }
                    if (field[i][j] == 0)
                        n++;
                }
        }
        
        return true;
    }
    
    public int getShortestChain(){
        return shortestChain;
    }

    public int getNumberOfColors(){
        return nOfColors;
    }
    
    public int getFieldSize(){
        return fieldSize;
    }

    public void setNumberOfColors( int n ){
        nOfColors = n;
    }
    
    public void setFieldSize(int _fieldSize) {
        int i, j;
        fieldSize = _fieldSize;
        field = new int [fieldSize][fieldSize];
        searchField = new int [fieldSize][fieldSize];
        for (i=0; i<fieldSize; i++)
            for (j=0; j<fieldSize; j++)
                field[i][i] = 0;
    }
    
    public void resetField(){
        setFieldSize( fieldSize );
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
    
}
