/*
 * LinesRecord.java
 *
 * Created on December 1, 2001, 3:12 PM
 */

/**
 *
 * @author  jplsp
 * @version 
 */

import java.util.*;
import java.io.*;

public class LinesRecord {
    
    private Vector names;
    private Vector scores;
    
    private int recordSize;
    private static final int defaultRecordSize = 10;
    private static final String defaultFileName = "jLines.score";
    
    /** Creates new LinesRecord */
    public LinesRecord() {
    }
    
    public int getRecordSize(){
        return recordSize;
    }

    public int getActualSize(){
        return names.size();
    }

    public String getNameAt( int i ){
        return (String)(names.elementAt(i));
    }
    
    public String getScoreAt( int i ){
        return (String)(scores.elementAt(i));
    }
    
    private void initFromScratch(){
        names = new Vector();
        scores = new Vector();
        recordSize = defaultRecordSize;
    }
    
    public void initFromFile() {
        BufferedReader in;
        int i, actualSize;

        names = new Vector();
        scores = new Vector();

        try {
            in = new BufferedReader( new FileReader( defaultFileName ) );
            
            recordSize = Integer.parseInt( in.readLine() );
            actualSize = Integer.parseInt( in.readLine() );
            
            for (i=0; i<actualSize; i++){
                names.add( in.readLine() );
                scores.add( in.readLine() );
                // to ensure that scores are in the right format
                Integer.parseInt((String)(scores.elementAt(scores.size() - 1)));
            }
        } catch (FileNotFoundException e) {
            System.err.println("!initFromFile: score file not found");
            initFromScratch();
        } catch (IOException e) {
            System.err.println("!initFromFile: could not read score file");
            initFromScratch();
        } catch (NumberFormatException e){
            System.err.println("!initFromFile: score file corupt");
            initFromScratch();
        }
    }
    
    public void sync(){
        PrintWriter out;
        String buf;
        int i;

        try {
            out = new PrintWriter( new java.io.FileOutputStream( defaultFileName ) );
            out.println(recordSize);
            out.println(scores.size());
        
            for (i=0; i<scores.size(); i++){
                out.println((String)names.elementAt(i));
                out.println((String)scores.elementAt(i));
            }
            
            out.close();
        } catch (FileNotFoundException e) {
            System.err.println("!sync: score file not found");
        }
    }
    
    public boolean newRecord(int _score){
        int i;
        if (scores.size() < recordSize)
            return true;
        for (i=0; i<names.size(); i++)
            if (Integer.parseInt((String)scores.elementAt(i)) < _score)
                return true;
        return false;
    }

    public void addRecord(int _score, String _name){
        int i;
        boolean added = false;
        for (i=0; i<names.size(); i++)
            if (Integer.parseInt((String)scores.elementAt(i)) < _score){
                names.add(i, _name);
                scores.add(i, Integer.toString(_score));
                added = true;
                break;
            }
        if (!added) {
            names.add(_name);
            scores.add(Integer.toString(_score));
        }
        if (names.size() > recordSize){
            names.removeElementAt(names.size() - 1);
            scores.removeElementAt(scores.size() - 1);
        }
    }
    
}
